// $Id: data.cpp,v 1.15 1998/08/31 16:36:01 lars Exp $
#include "data.h"

#include <kapp.h>
#include <ksimpleconfig.h>
#include <kmsgbox.h>

#include <qfileinf.h>
#include <qfile.h>
#include <qdict.h>
#include <qstring.h>

#include <unistd.h>
#include <sys/types.h>
#include <assert.h>
#include <stdlib.h>

#include "global.h"

Data *Data::dataobject = 0;

Data::Data()
{
    dataobject = this;
    mgettyoptions = 0;
    voiceoptions = 0;
    vgettyoptions = 0;
    sendfaxoptions = 0;

    init();
}

Data::~Data()
{
    delete mgettyoptions;
    delete voiceoptions;
    delete vgettyoptions;
    delete sendfaxoptions;
    dataobject = 0;
}

void
Data::init()
{
    // clean entries
    if(mgettyoptions != 0)  delete mgettyoptions;
    if(voiceoptions != 0)   delete voiceoptions;
    if(vgettyoptions != 0)  delete vgettyoptions;
    if(sendfaxoptions != 0) delete sendfaxoptions;

    mgettyoptions = new QDict<QString>(71, true, true);
    mgettyoptions->setAutoDelete(true);
    readFile(MGETTY);
    voiceoptions = new QDict<QString>(71, true, true);
    voiceoptions->setAutoDelete(true);
    vgettyoptions = new QDict<QString>(71, true, true);
    vgettyoptions->setAutoDelete(true);
    readFile(VOICE);
    sendfaxoptions = new QDict<QString>(71, true, true);
    sendfaxoptions->setAutoDelete(true);
    readFile(SENDFAX);
    
    readonlyflag = !testWritePermission();
    dirty = false;
}

void Data::sync() 
{ 
  dropPrivileges();
  cfg->sync(); 
  getPrivileges();
}

Data *
Data::getData()
{  
  if(dataobject == 0)
    dataobject = new Data;
  return dataobject; 
}

const QString Data::readMgettyEntry(const char *key)
{
  QString retval;
  QString *aStr = mgettyoptions->find(key);
  if(aStr == 0) 
    retval = 0;
  else 
    retval = aStr->copy();
  return retval;
}

const QString Data::readVoiceEntry(const char *key)
{
  QString retval;
  QString *aStr = voiceoptions->find(key);
  if(aStr == 0) 
    retval = 0;
  else 
    retval = aStr->copy();
  return retval;
}

const QString Data::readVgettyEntry(const char *key)
{
  QString retval;
  QString *aStr = vgettyoptions->find(key);
  if(aStr == 0) 
    retval = 0;
  else 
    retval = aStr->copy();
  return retval;
}

const QString Data::readSendfaxEntry(const char *key)
{
  QString retval;
  QString *aStr = sendfaxoptions->find(key);
  if(aStr == 0) 
    retval = 0;
  else 
    retval = aStr->copy();
  return retval;
}


void Data::writeMgettyEntry(const char *key, const char *value)
{
  QString *val = new QString(value);
  if(readMgettyEntry(key) != value)
  {
    mgettyoptions->replace(key,val);
    dirty = true;
  }
}

void Data::writeVoiceEntry(const char *key, const char *value)
{
  QString *val = new QString(value);
  if(readVoiceEntry(key) != value)
  {
    voiceoptions->replace(key,val);
    dirty = true;
  }
}

void Data::writeVgettyEntry(const char *key, const char *value)
{
  QString *val = new QString(value);
  if(readVgettyEntry(key) != value)
  {
    vgettyoptions->replace(key,val);
    dirty = true;
  }
}

void Data::writeSendfaxEntry(const char *key, const char *value)
{
  QString *val = new QString(value);
  if(readSendfaxEntry(key) != value)
  {
    sendfaxoptions->replace(key,val);
    dirty = true;
  }
}

void Data::writeBack()
{
  QString filename;
  int ok;

  if(readonlyflag || !dirty) return;

  ok = KMsgBox::yesNo(0, "kvoice", 
		      i18n("Do you want me to apply the changes\n"
			   "to the m/vgetty config files?") );
  debug("answer = %d",ok);
  if(ok != 2) //yes
  {
      // writing the config files
      writeFile(MGETTY);
      writeFile(VOICE);
      writeFile(SENDFAX);
  }

  // this is needed, because the dicts are destroyed while writing
  // or we wan't to restore the original state
  init();
}

bool Data::readonly()
{
  return readonlyflag;
}

void Data::readFile(int which)
{
  QString line, filename, key;
  QString *entry;
  QDict<QString> *dict;

  gcfg->setGroup("location");
  filename = gcfg->readEntry("mgettyDir").copy();
  if(which == MGETTY)
  {
    filename += "/mgetty.config";
    dict = mgettyoptions;
  }
  else if(which == VOICE)
  {
    filename += "/voice.conf";
    dict = 0;
  }
  else
  {
    filename += "/sendfax.config";
    dict = sendfaxoptions;
  }

  QFile file(filename);
  if(!file.exists()) 
  {
    debug("%s missing", filename.data());
    return;
  }
  if(!file.open(IO_ReadOnly))
  {
    debug("Could not open file %s",filename.data());
    return;
  }
  QTextStream aStream( &file );
  aStream.device()->at(0);
 
  while(!aStream.eof())
  {
    line = aStream.readLine();
    if( line.isEmpty() || line.find( "#" ) == 0 )
      continue;
    line = line.simplifyWhiteSpace();
    
    int pos = line.find(' ');
    key = line.left( pos );

    if( pos == -1 ) 
      entry = new QString("");
    else
      entry = new QString(line.right( line.length()-pos-1 ));
    debugAll("key=%s entry=%s",(const char *)key,
	     (const char *)*entry );

    if (which == VOICE && (key == "part" || key == "program"))
    {
      if(*entry == "generic")
	dict = voiceoptions;
      else if(*entry == "vgetty")
	dict = vgettyoptions;
      else 
	dict = 0;
      delete entry;
      continue;
    }
    
    if(dict != 0 && !key.isEmpty())
      dict->replace(key, entry);
    else
      delete entry;
  }
  file.close();
}

bool 
Data::writeFile(int which)
{
  // the m+vgetty configuration should only written back, if the user
  // has the privileges to do so...
  // 
  // the new file should be written such as to preserve as much as possible
  // of the old file (comments, etc...)
  QString line, key, oldvalue;
  QString filename;
  QDict<QString> *dict;

  gcfg->setGroup("location");
  filename = gcfg->readEntry("mgettyDir").copy();
  if(which == MGETTY)
  {
    filename += "/mgetty.config";
    dict = mgettyoptions;
  }
  else if(which == VOICE)
  {
    filename += "/voice.conf";
    dict = 0;
  }
  else
  {
    filename += "/sendfax.config";
    dict = sendfaxoptions;
  }

  dropPrivileges();

  QFile oldfile(filename); 
  if(!oldfile.open(IO_ReadWrite))
  {
      debug("could not write...");
      getPrivileges();
      return false;
  }


  gcfg->setGroup("location");
  QString newfilename = gcfg->readEntry("mgettyDir") + "/__kvoice__tmpfile";
  QFile newfile(newfilename);
  assert( newfile.open(IO_WriteOnly) );

  QTextStream aStream(&oldfile);

  while(!aStream.eof())
  {
    line = aStream.readLine();
    if( !line.isEmpty() && !(line.find( '#' ) == 0) )
    {
      line = line.simplifyWhiteSpace();
      
      int pos = line.find(' ');
      key = line.left( pos );
      if( pos == -1 ) 
	oldvalue = 0;
      else
	oldvalue = line.right( line.length()-pos-1 );
      
      if(which == VOICE && (key == "part" || key == "program"))
      {
	if(oldvalue == "generic")
	{
	  if(dict != 0)
	    writeRemainingEntries(&newfile, dict);
	  dict = voiceoptions;
	}
	else if(oldvalue == "vgetty")
	{
	  if(dict != 0)
	    writeRemainingEntries(&newfile, dict);
	  dict = vgettyoptions;
	}
	else 
	  dict = 0;
      }
      else if(dict != 0)
      {
	QString *value = dict->find(key);
	if (value != 0)
	{
	  line = key + " " + *value;
	  dict->remove(key);
	}
      }
    }
    if(!line.isEmpty())
      newfile.writeBlock( line, line.length() );
    newfile.putch('\n');
  }
  // now write the remaining entries in 
  if(which == VOICE)
  {
    if(!voiceoptions->isEmpty())
    {
      if(dict != voiceoptions)
	newfile.writeBlock("part generic\n", 13);
      writeRemainingEntries(&newfile,voiceoptions);
    }
    if(!vgettyoptions->isEmpty())
    {
      if(dict != vgettyoptions)
	newfile.writeBlock("program vgetty\n", 15);
      writeRemainingEntries(&newfile,vgettyoptions);
    }
  }
  else if(dict != 0)
    writeRemainingEntries(&newfile,dict);
    
  oldfile.close();
  newfile.close();

  // we finished writing, now we can move the new file to the right position
  QString oldfilename = filename + ".old";

  if(rename(filename,oldfilename) == 0)
      rename(newfilename,filename);
  else
  {
      rename(oldfilename,filename);
      unlink(newfilename);
      getPrivileges();
      return false;
  }

  getPrivileges();

  return true;
}

void
Data::writeRemainingEntries(QFile *file, QDict<QString> *dict)
{
  QDictIterator<QString> it(*dict);
  while ( it.current() ) {
    QString buffer;
    QString value = *it.current();
    buffer.sprintf("%s %s\n", it.currentKey(), (const char *)value );
    file->writeBlock( buffer, buffer.length() );
    ++it;
  }
}

bool Data::testWritePermission()
{
  QString aStr;

  dropPrivileges();

  gcfg->setGroup("location");
  aStr = gcfg->readEntry("mgettyDir") + "/mgetty.config";
  QFileInfo info(aStr);
  if(!info.exists())
    system("touch " + aStr);
  if(!info.isWritable()) 
  {
    getPrivileges();
    return false;
  }

  aStr = gcfg->readEntry("mgettyDir") + "/voice.conf";
  info.setFile(aStr);
  if(!info.exists())
    system("touch " + aStr);
  if(!info.isWritable())
  {
    getPrivileges();
    return false;
  }

  aStr = gcfg->readEntry("mgettyDir") + "/sendfax.config";
  info.setFile(aStr);
  if(!info.exists())
    system("touch " + aStr);
  if(!info.isWritable())
  {
    getPrivileges();
    return false;
  }

  debug("Got write permission to m/vgetty config files.");
  return true;
}

int Data::getModemSpeed()
{
  int speed;

  speed = readMgettyEntry("speed").toInt();
  if(speed == 0) speed = 38400;
  return speed;
}

//FIXME
const char *Data::getModemType()
{
  gcfg->setGroup("modem");
  return gcfg->readEntry("type");
}

//FIXME
const char *Data::getCompression() { return "4"; }

