// $Id: gettyhandler.cpp,v 1.12 1998/09/06 20:28:52 lars Exp $
#include "global.h"
#include "gettyhandler.moc"
#include "data.h"

#include <assert.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
#include <stdlib.h>
    
#include <kprocess.h>
#include <ksimpleconfig.h>
#include <kmsgbox.h>
#include <kapp.h>

#include <qstring.h>
#include <qfile.h>
#include <qtstream.h>

GettyHandler *GettyHandler::currentHandler = 0;

GettyHandler::GettyHandler()
  : QObject()
{
  if(currentHandler != 0)
    delete currentHandler;
  currentHandler = this;

  proc = 0;
}

GettyHandler::~GettyHandler()
{
  currentHandler = 0;

  if( proc == 0) return;
  delete proc;
}

bool
GettyHandler::start()
{
  debug("trying to start getty\n");
  return true;
  // check if vgetty is run by init
//  if(isRunFromInittab()) return true;

//  debug("vgetty is not run by inittab");
//  tries = 0;
  // ok vgetty is not run by init. Let's assume it doesn't run (FIXME)
//  restartGetty();
//  return true;
}

void 
GettyHandler::quit()
{
  if(proc == 0) return;

  proc->kill(SIGKILL);
  delete proc;
  proc = 0;
}

void 
GettyHandler::slotGettyHasExited(KProcess *)
{
  int delta;

  delta = time(0)-started_at;
  if(delta < 5) tries++;

  if(proc == 0) return;
  delete proc;

  if(tries > 3)
  {
    proc = 0;
    debug("error starting vgetty");
    KMsgBox::message(0, i18n("Warning"), 
		     i18n("An error occured while trying to\n"
			  "start vgetty. Maybe the options given\n"
			  "to kvoice aren't correct.\n"
			  "For detailed error messages see the file\n"
			  "/var/log/vgetty.your_modem_device"), 1);
    return;
  }
  restartGetty();
}

void
GettyHandler::restartGetty()
{
  QString executable;

  gcfg->setGroup("executables");
  executable = gcfg->readEntry("vgetty");
  printf("starting %s\n",executable.data());
  if(executable.isEmpty())
  {
    // no vgetty, will use mgetty for fax only
    executable = gcfg->readEntry("mgetty");
    if(executable.isEmpty()) return;
  }

  gcfg->setGroup("modem");
  QString device = gcfg->readEntry("device");
  if(device.isEmpty()) return;

  proc = new KProcess(); 
  proc->clearArguments();
  *proc << executable;
  *proc << device;
  connect(proc, SIGNAL(processExited(KProcess *)),
	  this, SLOT(slotGettyHasExited(KProcess *)) );

  started_at = time(0);
  proc->start();
}

bool 
GettyHandler::isRunFromInittab()
{
  QString device;
  device = deviceFromInittab();
  
  if(device.isEmpty())
    return false;
  return true;
}

const QString 
GettyHandler::deviceFromInittab()
{
  QString line;

  // can this be in other locations???
  QFile inittab("/etc/inittab"); 

  if(inittab.open(IO_ReadOnly) == false)
    debug("failed opening inittab");
  
  QTextStream aStream(&inittab);
  aStream.reset();

  while(!aStream.eof())
  {
    line = aStream.readLine();
    if(line.find("vgetty") != -1 || line.find("mgetty") != -1 )
    {
      if (line.find("#") == -1) // not 100% correct, but not bad... FIXME!
      {
	// device name should come last...
	int pos = line.findRev(" ") + 1;
	QString device = line.right(line.length()-pos);
	if(device.find("/dev/") == -1)
	  device.prepend("/dev/");
	//debug("device = %s",(const char *)device);
	inittab.close();
	return device;
      }
    }
  }
  //debug("device = 0");
  inittab.close();
  return 0;
}

const QString 
GettyHandler::getDevice(bool strip)
{
  QString initdevice = deviceFromInittab();
  gcfg->setGroup("modem");
  QString device = gcfg->readEntry("device").copy();
  
  if(!initdevice.isEmpty() && initdevice != device)
  {
    gcfg->writeEntry("device",initdevice);
    device = initdevice;
  }
  
  if(strip) // strip off the directory
  {
    int len = device.findRev("/");
    if(len != -1)
      device = device.right(device.length() - len - 1);
  }
  return device;
}

bool
GettyHandler::inittabEntryExists()
{
    QString initdevice = deviceFromInittab();
    if(!initdevice.isEmpty())
	return true;
    return false;
}

GettyHandler *
GettyHandler::getGettyHandler()
{
  if( currentHandler == 0)
    currentHandler = new GettyHandler;
  return currentHandler;
}

bool 
GettyHandler::writeInittab(QString device)
{
    bool found=false;
    bool flagoff = false;
    int ok;
    QString executable, aStr;

  // stop the running of vgetty
    if(proc != 0)
    {
	proc->kill();
	delete proc;
	proc = 0;
    }

    if(isSuid && getuid() != 0) return false;

    aStr = DATA->readVgettyEntry("answer_mode");
    gcfg->setGroup("executables");
    if(aStr.contains("voice"))
	executable = gcfg->readEntry("vgetty");
    else
	executable = gcfg->readEntry("mgetty");
    if(executable.isEmpty()) return false;

    dropPrivileges();

    QString line;

    // can this be in other locations???
    QFile inittab("/etc/inittab"); 

    if(!inittab.open(IO_ReadWrite)) {
	getPrivileges();
	return false;
    }

    if(device.isEmpty())
    {
	ok = KMsgBox::yesNo(0, i18n("kvoice information"),
			    i18n("Do you really wan't me to delete\n"
				 "the modem device from /etc/inittab?\n"
				 "Your modem will no be able to answer\n"
				 "incoming calls anymore."),
			    KMsgBox::QUESTION);
	if(ok == 2) // no
	{
	    getPrivileges();
	    return false;
	}
    }
    else
    {
	ok = KMsgBox::yesNo(0, i18n("kvoice information"),
			    i18n("Shall I write the new modem\n"
				 "device to /etc/inittab?\n"),
			    KMsgBox::QUESTION);
	debug("answer = %d",ok);
	if(ok == 2) // no
	{
	    // to be sure, that the gcfg object contains the right device
	    gcfg->setGroup("modem");
	    gcfg->writeEntry("device",getDevice());
	    getPrivileges();
	    return false;
	}
    }
    debug("now starting to write inittab\n");

    // we have to take care of the nologin file...
    QString olddevice = "/etc/nologin." + getDevice(true);
    QFile file(olddevice);
    if(file.exists())
	flagoff = true;
  
    gcfg->setGroup("location");
    QString filename = "/etc/inittab.kvoice";
    QFile newtab(filename);
    assert( newtab.open(IO_WriteOnly) );
  
    QTextStream aStream(&inittab);
  
    while(!aStream.eof())
    {
	line = aStream.readLine();
	if((line.find("vgetty") != -1 || line.find("mgetty") != -1) && !found )
	{
	    if(!device.isEmpty())
	    {
		found = true;
		// device name should come last...
		line = "mo:235:respawn:";
		line += (const char *)executable;
		if(device.find("/dev") == -1)
		    line += " /dev/";
		else 
		    line += " ";
		line += (const char *)device;
	    } 
	    else 
		if(line.find("#") == -1)
		    line = "# " + line;
	}
	if(line != "")
	    newtab.writeBlock( (const char *)line, line.length() );
	newtab.putch('\n');
    }
    if(!found && !device.isEmpty())
    {
	// we'll have to write a new entry...
	line = "\n# the following entry was generated by kvoice\n";
	newtab.writeBlock( (const char *)line, line.length() );

	line = "mo:235:respawn:";
	line += (const char *)executable;
	if(device.find("/dev") == -1)
	    line += " /dev/";
	else
	    line += " ";
	line += device.copy();
	newtab.writeBlock( (const char *)line, line.length() );
	newtab.putch('\n');
    }
    newtab.close();
    inittab.close();

    if(rename("/etc/inittab","/etc/inittab.old") == 0)
    {
	if(rename(filename,"/etc/inittab") != 0)
	{
	    debug("unable to move new inittab");
	    rename("/etc/inittab.old","/etc/inittab");
	    return false;
	}
	sync();
	usleep(500000);
    }
    else
    {
	debug("unable to move old inittab");
	unlink(filename);
	return false;
    }
	
    getPrivileges();

    if(flagoff)
    {
	off();
	unlink(olddevice);
    }


    kill(1,SIGHUP); // force init to reread the inittab
    //start(); // restart vgetty if we need to
    
    return true;
}

void
GettyHandler::on()
{
    QString aStr;
    aStr = "/etc/nologin.";
    aStr += getDevice(true);

    unlink(aStr);

    // do we need to send a signal to vgetty?
}

void
GettyHandler::off()
{
    QString aStr;
    aStr = "/etc/nologin.";
    aStr += getDevice(true);

    QFile file(aStr);
    file.open(IO_WriteOnly);
    file.writeBlock("off",3);
    file.close();

    // do we need to send a signal to vgetty?
}

void
GettyHandler::slotTakeCall()
{
    //pid_t pid;
    //FIXME: need to find the process ID of vgetty
    // and then send a signal USR1 to it

    // works at least with Linux ...
    system("killall -q -USR1 vgetty");
    system("killall -q -USR1 mgetty");

    //kill(pid, SIGUSR1);
}
