// $Id: greetings.cpp,v 1.13 1998/08/31 16:36:01 lars Exp $
#include <qdir.h>
#include "greetings.moc"

#include "data.h"
#include "messages.h"
#include "kvfiledlg.h"
#include "messagelist.h"
#include "soundio.h"

#include <unistd.h>

#include <kmsgbox.h>
#include <kiconloader.h>
#include <kmenubar.h>
#include <kstdaccel.h>
#include <qlayout.h>
#include <qstring.h>
#include <qregexp.h>
#include <qpopmenu.h>

#include "global.h"

Greetings::Greetings(const char *name,
		     Messages  *m)
  : KTopLevelWidget(name)
{
  data = Data::getData();
  messages = m;
  greetingDir = 0;

  setupMenuBar();
  setupToolBar();
  init();

  initNewDialog();
  readConfig();

}

Greetings::~Greetings()
{
  writeConfig();

  delete greetingDir;
  delete recorddialog;
}

void
Greetings::readConfig()
{
  QString str;
  int w,h;

  setMinimumSize(200, 100);
  cfg->setGroup("geometry");
  str = cfg->readEntry("greetings", "250 300");
  sscanf(str.data(), "%d %d", &w, &h);
  resize(w, h);

  str = cfg->readEntry("greetings-new", "300 350");
  sscanf(str.data(), "%d %d", &w, &h);
  recorddialog->resize(w, h);
}

void
Greetings::writeConfig(bool sync)
{
  QString str;

  cfg->setGroup("geometry");
  str.sprintf("%d %d", width(), height());
  cfg->writeEntry("greetings", str);

  str.sprintf("%d %d", recorddialog->width(), recorddialog->height());
  cfg->writeEntry("greetings-new", str);

  if(sync) cfg->sync();
}

void
Greetings::setupMenuBar()
{
  KMenuBar *menubar = new KMenuBar(this);

  QPopupMenu *file = new QPopupMenu();
  file->insertItem( i18n("New"), this, SLOT(slotNew()) );
  file->insertItem( i18n("Play") , this, SLOT(slotPlay()) );
  file->insertItem( i18n("Stop") , this, SLOT(slotStop()) );
  file->insertItem( i18n("Delete") , this, SLOT(slotDelete()) );
  file->insertSeparator();
  file->insertItem( i18n("&Close"), this, SLOT(slotFinished()), keys->close() );
  menubar->insertItem(i18n("&File"),file);

  menubar->insertSeparator();
  QPopupMenu *help = new QPopupMenu();
  help->insertItem( i18n("Help"), this, SLOT(slotHelp()) );
  menubar->insertItem(i18n("&Help"),help);

  setMenu(menubar);
}

void
Greetings::setupToolBar()
{
  KIconLoader* loader = kapp->getIconLoader();

  toolbar = new KToolBar(this);

  toolbar->insertButton(loader->loadIcon("filenew2.xpm"),BUTTON_NEW,
			SIGNAL(clicked()),this,
			SLOT(slotNew()),TRUE,i18n("New Message"));
  toolbar->insertButton(loader->loadIcon("kvplay.xpm"),BUTTON_PLAY,
			SIGNAL(clicked()),this,
			SLOT(slotPlay()),TRUE,i18n("Play Message"));
  setPlay(false);
  toolbar->insertButton(loader->loadIcon("kvstop.xpm"),BUTTON_STOP,
			SIGNAL(clicked()),this,
			SLOT(slotStop()),TRUE,i18n("Stop playing"));
  setStop(false);
  toolbar->insertButton(loader->loadIcon("filedel2.xpm"),BUTTON_DEL,
			SIGNAL(clicked()),this,
			SLOT(slotDelete()),TRUE,i18n("Delete Message"));
  toolbar->insertSeparator();
  toolbar->insertButton(loader->loadIcon("exit.xpm"),BUTTON_CLOSE,
			SIGNAL(clicked()),this,
			SLOT(slotFinished()),TRUE,i18n("Close"));

  connect(SOUNDIO, SIGNAL(playing()), 
	  this, SLOT(slotMessageIsPlayed()) );
  connect(SOUNDIO, SIGNAL(finished()), 
	  this, SLOT(slotPlayingFinished()) );

  addToolBar(toolbar);
}

void
Greetings::init()
{
  view = new QWidget(this);
  setView(view);

  QBoxLayout *layout = new QHBoxLayout(view,4);

  greetings = new MessageList(0, view, "greetings");
  greetings->setColumn(1, i18n("Name"), greetings->columnWidth(1) );
  greetings->setColumn(3, i18n("Created"), greetings->columnWidth(3) );
  layout->addWidget(greetings,1);

  connect(greetings,SIGNAL(highlighted(int, int)),
	  this,SLOT(slotMessageHighlighted(int, int)) );
  connect(greetings,SIGNAL(selected(int, int)),
	  this,SLOT(slotMessageSelected(int, int)) );

  getGreetings();

  dropZone = new KDNDDropZone(greetings, DndURL);
  connect(dropZone, SIGNAL(dropAction(KDNDDropZone *)), 
	  SLOT(slotDrop()) );
}

void
Greetings::initNewDialog()
{
  recorddialog = new QWidget();
  recorddialog->setCaption(i18n("New greeting message"));
  recorddialog->setMinimumSize(250,250);

  QBoxLayout *reclayout = new QVBoxLayout(recorddialog,5);
  QLabel *text1 = new QLabel(i18n("Import existing sound file:")
			     ,recorddialog);
  text1->setMargin(0);
  text1->setMinimumSize(text1->sizeHint());
  reclayout->addWidget(text1);
  
  filedlg = new KVFileDlg(recorddialog);
  filedlg->setMinimumSize(100,100);
  reclayout->addWidget(filedlg,1);
  connect(filedlg, SIGNAL(fileSelected(const char *)), 
	  this, SLOT(slotImportFile(const char *)) );
  connect(filedlg, SIGNAL(canceled()),
	  this, SLOT(slotNewFinished()) );
  
  QFrame *frame1 = new QFrame(recorddialog);

  frame1->setMinimumSize(0,2);
  frame1->setMaximumSize(10000,2);
  frame1->setFrameStyle(QFrame::HLine | QFrame::Plain);
  frame1->setLineWidth(2);
  reclayout->addWidget(frame1);
  
  QLabel *text2 = new QLabel(i18n("Record a message with local headset:"), 
			     recorddialog);
  text2->setMargin(0);
  text2->setMinimumSize(text2->sizeHint());
  reclayout->addWidget(text2);

  QBoxLayout *layout2 = new QHBoxLayout();
  reclayout->addLayout(layout2);

  entergreeting = new QLineEdit(recorddialog);
  entergreeting->setMinimumSize(80,20);
  entergreeting->setMaximumSize(500,20);
  connect(entergreeting, SIGNAL(returnPressed()), 
	  this, SLOT(slotRecFromHeadset()) );
  layout2->addWidget(entergreeting);

  layout2->addStrut(20);
  
  QPushButton *recordbutton = new QPushButton(i18n("Record"),
					      recorddialog,"record");
  recordbutton->setMinimumSize(recordbutton->sizeHint());
  recordbutton->setMaximumSize(recordbutton->sizeHint());  
  layout2->addWidget(recordbutton);
  connect(recordbutton,SIGNAL(clicked()), this, SLOT(slotRecFromHeadset()) );
}

void Greetings::getGreetings()
{
  QString aStr;

  if(greetingDir != 0)
    delete greetingDir;
  gcfg->setGroup("location");
  greetingDir = new QDir(gcfg->readEntry("voiceMessages"));
  greetingDir->setFilter(QDir::Files);

  greetings->clear();

  aStr = data->readVoiceEntry("message_list");
  if(aStr.isEmpty())
    aStr = "Index";
  indexfilename  = greetingDir->path();
  indexfilename += "/";
  indexfilename += aStr;
  makeIndex();

  QFile *index = new QFile(indexfilename);
  if(!index->open(IO_ReadOnly))
    {
      KMsgBox::message(0, i18n("Warning"),
		       i18n("Could not get Index file"));
      return;
    }

  char line[128];
  while(1)
    {
      if(index->readLine(line,128) <= 0) break;
      aStr.sprintf(" \n%s",line);
      greetings->insertItem(aStr);
    }
  index->close();
  delete index;
}

void Greetings::slotFinished()
{
  emit finished();
}
  
void Greetings::slotPlay()
{
  if(greetings->currentItem() == -1) return;

  QString aFileName;
  aFileName=greetingDir->path();
  aFileName+="/";
  aFileName+=greetings->text(greetings->currentItem(), 1);

  debug("playing %s\n",(const char *)aFileName);
  SOUNDIO->play(aFileName);
}

void Greetings::slotStop()
{
  messages->slotStop();
}

void Greetings::slotDelete()
{
  if(greetings->currentItem() == -1) return;

  int yes = KMsgBox::yesNo(0,i18n("Delete Voice Mail"),
			   i18n("Are you sure, you\n"
			       "want to delete this message?") );
  if(yes == 1)
    {
      //remove file
      QString aFileName;
      aFileName = greetingDir->path();
      aFileName += "/";
      aFileName += greetings->text(greetings->currentItem(), 1);
      debug("deleting %s\n",(const char *)aFileName);
      if(!greetingDir->remove( (const char *)aFileName ))
      	debug("couldn't delete\n");

      getGreetings();
    }
}

void Greetings::slotNew()
{
  hide();
  recorddialog->show();
}

void Greetings::slotNewFinished()
{
  show();
  recorddialog->hide();
}

void 
Greetings::slotImportFile(const char *name)
{
  //get extension and base
    debug("filename = %s",name);
  QString filename;
  filename = strdup(name);
  QFile file(filename);
  if(!file.exists())
  {
     KMsgBox::message(0,i18n("Error importing file"),
		      i18n("The filename you specified\n"
			   "does not exist."),
		      KMsgBox::EXCLAMATION);
     return;
  }
  int index  = filename.findRev('.');
  int index2 = filename.findRev('/');
  if(index == -1)
  {
      KMsgBox::message(0,i18n("Error importing file"),
		      i18n("Unknown file format"),
		      KMsgBox::EXCLAMATION);
      return;
  }

  gcfg->setGroup("location");
  QString greetingfile = gcfg->readEntry("voiceMessages");

  greetingfile = greetingfile + filename.mid(index2, index-index2);

  SOUNDIO->convertToRmd(filename, greetingfile);
  getGreetings();
  slotNewFinished();
}

// records a greeting message
void Greetings::slotRecFromHeadset()
{
  int yes;

  recorddialog->hide();

  QString file=entergreeting->text();
  if(file.isEmpty()) return;
  file =  "/" + file;
  file = greetingDir->path() + file;
  debug("%s\n",(const char *)file);

  cfg->setGroup("voice");
  QString device = cfg->readEntry("device");
  if(device == "Headset" || device.isEmpty())
    yes = KMsgBox::yesNo(0,i18n("Record the Greeting message"), 
			 i18n("Lift up your telphone and\n"
			      "start talking after the signal.\n\n"
			      "Hang up the phone after you"
			      " finished your message."), 
			 0, i18n("Start"), i18n("Cancel") );
  else
    yes = KMsgBox::yesNo(0,i18n("Record the Greeting message"), 
			 i18n("Record a message from the modems"
			      "internal microphone."),
			 0, i18n("Start"), i18n("Cancel") );

  if ( yes == 1 ) {
    slotNewFinished();
	  
    proc = new KProcess();
    proc->setExecutable("vm");
    *proc << "record";
    if(device == "Internal")
	*proc << "-i";
    else
	*proc << "-H" << "-w";

    *proc << (const char *)file;
    connect(proc, SIGNAL(processExited(KProcess *)), 
	    this, SLOT(slotRecordFinished(KProcess *)) );
    connect(proc, SIGNAL(receivedStdout(KProcess *, char *, int)),
	    this, SLOT(slotRecordStdout(KProcess *, char *, int)) );
    connect(proc, SIGNAL(receivedStderr(KProcess *, char *, int)),
	    this, SLOT(slotRecordStdout(KProcess *, char *, int)) );
    // fixme: setup communication
    proc->start(KProcess::NotifyOnExit, KProcess::AllOutput);
  }
}

void 
Greetings::slotRecordStdout(KProcess *, char * buffer, int buflen)
{
  QString buf = buffer;
  buf.truncate(buflen);
  debug("in recordStdout\n");
  fprintf(stdout,"%s",(const char *)buf);
}

void 
Greetings::slotRecordFinished(KProcess *proc)
{
  debug("finished recording the message.\n");
  delete proc;
  getGreetings();
}

void
Greetings::slotDrop()
{
  QString  file;
  QStrList *urlList;

  urlList = &dropZone->getURLList();

  for (file=urlList->first(); !file.isEmpty() ; file=urlList->next())
  {
       printf("url = %s\n",(const char *)file);
    file.replace("file:","");
    slotImportFile(file);
  }
}

void
Greetings::slotHelp()
{
  kapp->invokeHTMLHelp("kvoice/index-4.html#sec:greetings",0);
}

void
Greetings::slotMessageIsPlayed()
{
  setPlay(false);
  setStop(true);
}

void
Greetings::slotPlayingFinished()
{
  setPlay(true);
  setStop(false);
}

void 
Greetings::slotMessageHighlighted(int, int)
{
  if(!SOUNDIO->isPlaying())
    setPlay(true);
}

void 
Greetings::slotMessageSelected(int, int)
{
  slotPlay();
}

void
Greetings::setPlay(bool stat) { 
  toolbar->setItemEnabled(BUTTON_PLAY,stat); 
}

void
Greetings::setStop(bool stat) { 
  toolbar->setItemEnabled(BUTTON_STOP,stat); 
}

void
Greetings::setDelete(bool stat) { 
  toolbar->setItemEnabled(BUTTON_DEL,stat); 
}


bool 
Greetings::makeIndex()
{
  unsigned int i;

  QFile *index = new QFile(indexfilename);

  if(!index->exists())
    {
      QString string;
      string.sprintf(i18n("vgetty needs a file named\n %s \n"), 
		     (const char *)indexfilename);
      string +=i18n("Do you want me to create this file now?");
      int yes = KMsgBox::yesNo(0,i18n("Greeting Messages index file"),
				   (const char *) string);
      if(yes != 1) return false;
    }

  if(index->open(IO_WriteOnly))
  {
     debug("number of files in greeting dir %d",greetingDir->count());
     for(i=0;i<greetingDir->count();i++)
     {
	  QString file = (*greetingDir)[i];
	  if(strcmp(file,"Index"))
	  {
	       debug("%s",(const char *)file);
	       file += "\n";
	       index->writeBlock((const char *)file, file.length());
	  }
     }
     index->close();
     delete index;
  }
  else
  {
       QString message;
       message.sprintf(i18n("Could not write to file %s.\n"
			    "Please check the permissions of that file,\n"
			    "or run kvoice suid"),
		       (const char *)indexfilename);
       KMsgBox::message(0, i18n("Warning"),
			message);
  }
  return true;
}

