// $Id: kvconfig.cpp,v 1.16 1998/09/06 20:28:52 lars Exp $
#include "kvconfig.moc"
#include "global.h"
#include "data.h"

#include <unistd.h>

#include <qdir.h>

#include <kmsgbox.h>
#include <klocale.h>
#include <kapp.h>
#include <qregexp.h>

#include "defaults.h"

KVConfig::KVConfig(QWidget *parent, const char *name)
  : QObject(parent,name)
{
  QString aStr;
  if(isSuid)
       aStr = kapp->kde_configdir().copy();
  else
  {
       aStr = kapp->localkdedir().copy();
       testDir(aStr);
       aStr += "/share";
       testDir(aStr);
       aStr += "/config";
       testDir(aStr);
  }
  aStr += "/kvoicerc";
  config = new KSimpleConfig(aStr);
}

KVConfig::~KVConfig()
{
  gcfg->sync();
  config->sync();
  delete config;

  // update the global cfg object
  cfg->reparseConfiguration();
}

// static methods

int
KVConfig::setup(bool upgrade)
{
    if(upgrade)
	KMsgBox::message(0,0,i18n("You upgraded to a new version of kvoice.\n"
				  "Rechecking your systems configuration.\n"
				  "Please check it in options->setup.")
		   ,KMsgBox::EXCLAMATION);
    else
	KMsgBox::message(0,0,i18n("kvoice hasn't been run before.\n"
				  "It will now try to guess your\n"
				  "systems configuration.\n"
				  "Please check it "
				  "in options->setup.")
		   ,KMsgBox::EXCLAMATION);
  
  KVConfig *conf = new KVConfig();
  conf->success = true;

  conf->lookForExecutables();
  conf->findLocations();
  conf->setupModem();
  if(conf->success)
    conf->configured();
    
  delete conf;
  return 0;
}

// private methods

QString 
KVConfig::findDir(const char *root, const char **list)
{
  int i=0;
  QDir dir;

  while( !(list[i] == 0) )
  {
    QString path = root;
    path = path + '/'; 
    path = path + list[i];
    path.replace("//","/");
    debug("looking for %s...",(const char *)path);
    if(dir.cd(path)) 
    {
      debug("found.");
      return path;
    }
    i++;
  }
  debug("Couldn't find directory in stringlist.");
  success= false;
  return 0;
}

QString 
KVConfig::findDir(const char **roots, const char **list)
{
  int i=0;
  while ( !(roots[i] == 0) )
  {
    QString dir;
    if((dir = findDir(roots[i], list)) != 0)
      return dir;
    i++;
  }
  success = false;
  return 0;
}

QString 
KVConfig::findFile(const char *directory, const char **list)
{
  int i=0;
  QDir dir;
  while( !(list[i] == 0) )
  {
    QString file = directory;
    file = file + '/';
    file = file + list[i];
    file.replace("//","/");
    debug("looking for %s...",(const char *)file);
    if( dir.exists(file) )
    {
      debug("found.");      
      return file;
    }
    i++;
  }
  success= false;
  return 0;
}

QString 
KVConfig::askForDir(const char *message)
{
  //FIXME
  KMsgBox::message(0,i18n("kvoice Warning"),
		   message, KMsgBox::EXCLAMATION);
  return "";
  //success = false;
}

void
KVConfig::findLocations()
{
  QString aStr;
  QString spool = "";
  QString voice,fax;

  gcfg->setGroup("location");

  // find mgetty directory
  aStr = findDir("", Defaults::mgettydir);
  if(aStr.isEmpty())
    aStr = askForDir(i18n("could not find the mgetty directory.\n"
			  "(normally /etc/mgetty+sendfax)\n"
			  "Please enter the correct location\n"
			  "in the setup dialog."));
  gcfg->writeEntry("mgettyDir",aStr);

  // find spool directory.
  spool = findDir("", Defaults::spooldir);
  if(spool.isEmpty())
    spool = askForDir(i18n("could not find the spool directory.\n"
			   "Please enter the correct location\n"
			   "in the setup dialog."));
  gcfg->writeEntry("spoolDir",spool);

  // find voice directory
  voice = DATA->readVoiceEntry("voice_dir");
  if(voice.isEmpty())
  {
    voice = findDir(spool, Defaults::voicespool);
    if(voice.isEmpty())
      voice = askForDir(i18n("could not find directory, where vgetty\n"
			     "stores the voice files" 
			     "(normally /usr/spool/voice).\n"
			     "Please enter the correct location\n"
			     "in the setup dialog."));
    else
    {
      gcfg->writeEntry("voiceDir",voice);
      DATA->writeVoiceEntry("voice_dir",voice);
    }
  }
  else
      gcfg->writeEntry("voiceDir",voice);

  voice += "/";

  aStr = DATA->readVoiceEntry("receive_dir");
  if(aStr.isEmpty())
  {
    aStr = findDir(voice, Defaults::incoming);
    if(aStr.isEmpty())
      aStr = askForDir(i18n("could not find directory, where vgetty\n"
			    "stores incoming messages.\n"
			    "Please enter the correct location\n"
			    "in the setup dialog."));
    else
    {
      gcfg->writeEntry("voiceIncoming",aStr);
      int i = aStr.findRev("/");
      DATA->writeVoiceEntry("receive_dir",aStr.right(aStr.length()-i));
    }
  }
  else
      gcfg->writeEntry("voiceIncoming",voice + aStr);


  aStr = DATA->readVoiceEntry("message_dir");
  if(aStr.isEmpty())
  {
    aStr = findDir(voice, Defaults::messages);
    if(aStr.isEmpty())
      aStr = askForDir(i18n("could not find directory, where vgetty\n"
			    "stores the messages.\n"
			    "(normally /usr/spool/voice/messages)\n"
			    "Please enter the correct location\n"
			    "in the setup dialog."));
    else
    {
      gcfg->writeEntry("voiceMessages",aStr);
      int i = aStr.findRev("/");
      DATA->writeVoiceEntry("message_dir",aStr.right(aStr.length()-i));
    }
  }
  else
      gcfg->writeEntry("voiceMessages",voice + aStr);


  if( aStr = DATA->readVoiceEntry("message_list").isEmpty() )
    DATA->writeVoiceEntry("message_list","Index");
        
  // fax spool
  fax = findDir(spool, Defaults::faxspool);
  if(fax.isEmpty())
    fax = askForDir(i18n("could not find directory, where vgetty\n"
			 "stores the faxes.\n"
			 "(normally /usr/spool/fax)\n"
			 "Please enter the correct location\n"
			 "in the setup dialog."));
  gcfg->writeEntry("faxDir",fax);

  aStr = findDir(fax, Defaults::incoming);
  if(aStr.isEmpty())
    aStr = askForDir(i18n("could not find directory, where vgetty\n"
			  "stores the incoming faxes.\n"
			  "(normally /usr/spool/fax/incoming)\n"
			  "Please enter the correct location\n"
			  "in the setup dialog."));
  gcfg->writeEntry("faxIncoming",aStr);

  aStr = findDir(fax, Defaults::outgoing);
  if(aStr.isEmpty())
    aStr = askForDir(i18n("could not find directory, where vgetty\n"
			  "stores the outgoing faxes.\n"
			  "(normally /usr/spool/fax/outgoing)\n"
			  "Please enter the correct location\n"
			  "in the setup dialog."));
  gcfg->writeEntry("faxOutgoing",aStr);

  gcfg->writeEntry("tmpDir","/tmp");
}

void
KVConfig::configured()
{
  gcfg->setGroup("kvoice");
  gcfg->writeEntry("configured","yes");
  gcfg->writeEntry("version",VERSION);
  DATA->writeBack();

}

void 
KVConfig::setupModem()
{
  // FIXME
}

bool
KVConfig::lookForExecutables()
{
  QString aStr;
  bool retval = false;

  gcfg->setGroup("executables");

  aStr = findExecutable("mgetty");
  gcfg->writeEntry("mgetty",aStr);
  if(aStr.isEmpty()) retval = true;

  aStr = findExecutable("vgetty");
  gcfg->writeEntry("vgetty",aStr);
  if(!aStr.isEmpty()) retval = true;

  if(!retval) 
  {
    success = false;
    KMsgBox::message(0,"Warning",
		     i18n("m/vgetty not found. Without this package,\n"
			  "you will not be able to receive\n"
			  "fax or voice mails. You can get it from\n"
			  "http://www.leo.org/~doering/mgetty/")
		     ,KMsgBox::EXCLAMATION);
  }

  //aStr = findExecutable("hylafax");
  //gcfg->writeEntry("hylafax",aStr);
  // no hylafax support for now...
  //  if(!aStr.isEmpty()) retval = true;

  retval = true;

  aStr = findExecutable("faxspool");
  gcfg->writeEntry("faxspool",aStr);
  if(aStr.isEmpty()) retval = false;

  aStr = findExecutable("sendfax");
  gcfg->writeEntry("sendfax",aStr);
  if(aStr.isEmpty()) retval = false;

  aStr = findExecutable("faxrunq");
  gcfg->writeEntry("faxrunq",aStr);
  if(aStr.isEmpty()) retval = false;

  aStr = findExecutable("faxq");
  gcfg->writeEntry("faxq",aStr);
  if(aStr.isEmpty()) retval = false;

  aStr = findExecutable("faxrm");
  gcfg->writeEntry("faxrm",aStr);
  if(aStr.isEmpty()) retval = false;

  aStr = findExecutable("faxrunqd");
  gcfg->writeEntry("faxrunqd",aStr);
  if(aStr.isEmpty()) retval = false;

  return retval;
}

QString
KVConfig::findExecutable(QString executable)
{
  // get path
  QString path;
  QStrList pSearchPaths;
  int index = 0;
  int lastindex = 0;
  
  debug("searching for %s ...",(const char *)executable);

  path = getenv("PATH");
  while((index = path.find(":",lastindex+1)) != -1)
  {
    pSearchPaths.append(path.mid(lastindex+1,index-lastindex-1));
    lastindex = index;
  }
  if(lastindex != (int)path.size() - 2)
    pSearchPaths.append( path.mid(lastindex+1,path.size()-lastindex-1) );

  pSearchPaths.append("/sbin");
  pSearchPaths.append("/usr/sbin");
  pSearchPaths.append("/usr/local/sbin");
  pSearchPaths.append("/usr/local/bin");

  QStrListIterator it(pSearchPaths);

  while ( it.current() )
  {
    path = it.current();
    path += "/";
    path += executable;
    if ( !access( path, X_OK ) )
    {
      debug(" ... found");
      return path;
    }
    ++it;
  }
  debug(" ... missing");
  return "";                                
}
