// $Id: main.cpp,v 1.15 1998/08/31 16:36:01 lars Exp $
#include "mainwidget.h"
#include "global.h"
#include "kvconfig.h"
#include "defaults.h"

#include <stdio.h>
#include <stdarg.h>      
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>

#include <kapp.h>
#include <klocale.h>
#include <kstdaccel.h>
#include <kapp.h>
#include <kconfig.h>
#include <ksimpleconfig.h>
#include <kmsgbox.h>

// sound support
extern "C" {
#include <mediatool.h>
}
#ifndef _KAUDIO_H_
#define _KAUDIO_H_
#include "kaudio.h"
#endif

KApplication* app = 0;
KLocale* NLS = 0;
KConfig *cfg = 0;
KStdAccel* keys = 0;
KSimpleConfig *gcfg = 0;
KAudio *KAUDIO = 0;
KStatusBar *STATUSBAR = 0;

bool isSuid = false;
static uid_t eff_uid;
static gid_t eff_gid;

const char *aboutText = 
"kvoice v" VERSION "\n"
"written by Lars Knoll (knoll@mpi-hd.mpg.de)\n"
"This program is covered by the GPL.";

void mywarning(const char *msg, ...)
{
    va_list ap;
    va_start( ap, msg );                        // use variable arg list
    QString mess;
    mess.sprintf(msg, ap );         
    va_end( ap );
    KMsgBox::message(0,0,mess,KMsgBox::EXCLAMATION);
}    
   
/*
void warning(const char *msg, ...)
{
    va_list ap;
    va_start( ap, msg );                        // use variable arg list
    QString mess;
    mess.sprintf(msg, ap );         
    va_end( ap );
    KMsgBox::message(0,0,mess,KMsgBox::EXCLAMATION);
}       
*/

extern void dropPrivileges()
{
  seteuid(getuid());
  setegid(getgid());
}

extern void getPrivileges()
{
  seteuid(eff_uid);
  setegid(eff_gid);
}

void setup(bool upgrade = false)
{
  if(getuid() != geteuid())
  // we are running suid, and aren't root
  {
      if(upgrade)
	  KMsgBox::message(0,0,i18n("You upgraded to a new version of kvoice.\n"
				    "Please run kvoice as root to check\n"
				    "your configuration or ask you\n"
				    "system administrator to do so.")
			   ,KMsgBox::EXCLAMATION);
      else
	  KMsgBox::message(0,0,i18n("kvoice is not configured.\n"
				    "Run kvoice as root to configure it,\n"
				    "or ask your system administrator.")
			   ,KMsgBox::EXCLAMATION);
    exit(1);
  }
  // o.k. lets set things up.
  KVConfig::setup(upgrade);

}

void testDir( const char *_name )
{
  DIR *dp;
  dp = opendir(_name);
  if (dp == NULL)
  {
    mkdir(_name, S_IRWXU);
    chmod(_name, S_IRWXU || S_IRGRP || S_IXGRP || S_IROTH || S_IXOTH);
  }
  else
    closedir(dp);
}


int main( int argc, char **argv )
{
  // do we have root permissions?
  debug("uid = %d  euid = %d",getuid(), geteuid());
  isSuid = false;
  if(geteuid() == 0)
    isSuid = true;

  // save egid and euid
  eff_gid = getegid();
  eff_uid = geteuid();

  // this is needed, because otherwise some directories and files in ~/.kde
  // would be created as root
  dropPrivileges();
  app = new KApplication (argc, argv, "kvoice");
  getPrivileges();

  //FIXME: command line processing

  NLS = app->getLocale();
  cfg = app->getConfig();
  keys = new KStdAccel(cfg);

  // make sure the directory exists
  QString aStr;
  if(isSuid)
    aStr = app->kde_datadir().copy();
  else
  {
    aStr = app->localkdedir().copy();
    testDir(aStr);
    aStr += "/share";
    testDir(aStr);
    aStr += "/apps";
  }
  testDir(aStr);
  aStr += "/kvoice";
  testDir(aStr);

  aStr += "/config";
  printf("gcfg=%s\n",(const char *)aStr);
  gcfg = new KSimpleConfig(aStr);
  gcfg->setGroup("kvoice");
  if(gcfg->readEntry("configured") != "yes")
      setup();
  if(gcfg->readEntry("version") != VERSION)
      setup(true);

  KAUDIO = new KAudio();
  if (KAUDIO->serverStatus() != 0)
    debug( "Failed contacting audio server");
  else
    KAUDIO->setSignals(true);

  MainWidget *toplevel = new MainWidget();
  if (app->isRestored())
  { 
    // FIXME: DO SESSION MANGEMENT !!!!!
    if (KTopLevelWidget::canBeRestored(1))
      toplevel->restore(1);
  }

  app->setMainWidget( toplevel );
  toplevel->show();
  return app->exec();

}


