// $Id: mainwidget.cpp,v 1.19 1998/08/30 18:34:02 lars Exp $
#include "mainwidget.moc"

#include "global.h"
#include "modem.h"
#include "messages.h"
#include "data.h"
#include "setup.h"
#include "sendfax.h"
#include "greetings.h"
#include "gettyhandler.h"
#include "showfaxqueue.h"

#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <kapp.h>
#include <kcolordlg.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kstdaccel.h>
#include <kprocess.h>
#include <kmsgbox.h>

#include <qwidget.h>
#include <qpushbt.h>
#include <qfont.h>
#include <qmsgbox.h>
#include <qstring.h>
#include <qtabdlg.h>
#include <qkeycode.h>
#include <qfiledlg.h>
#include <qfile.h>
#include <qstrlist.h>
#include <kapp.h>


MainWidget *MainWidget::mainObject = 0L;

MainWidget::MainWidget( const char *name)
  : KTopLevelWidget(name)
{
  //initialize Objects
  DATA;

  sendfax = 0;
  greetings = 0;

  mainObject = this;
  
  readConfig();

  setupMessages();

  setupMenuBar();
  setupToolBar();
  setupStatusBar();

  //GETTYHANDLER->start();
}

MainWidget::~MainWidget()
{
  writeConfig(true);

  delete menubar;
  delete STATUSBAR;
  delete messages;
}

// private functions
void 
MainWidget::readConfig()
{
  QString str;
  int w,h;

  setMinimumSize(400,300);
  cfg->setGroup("geometry");
  str = cfg->readEntry("main", "550 400");
  sscanf(str.data(), "%d %d", &w, &h);
  resize(w, h);
}

void 
MainWidget::writeConfig(bool sync)
{
  QString str;

  cfg->setGroup("geometry");
  str.sprintf("%d %d", width(), height());
  cfg->writeEntry("main", str);

  if (sync) cfg->sync();
}

void MainWidget::setupMenuBar()
{
  menubar = new KMenuBar(this);

  // menu: file
  QPopupMenu *file = new QPopupMenu();
//  file->insertSeparator();
  file->insertItem( i18n("&Quit"), this, SLOT(slotQuit()), keys->quit() );
  menubar->insertItem(i18n("&File"), file);

  // menu: voice
  QPopupMenu *voice = new QPopupMenu();
  voice->insertItem( i18n("&Greeting message"), this, SLOT(slotGreeting()) );
  voice->insertItem( i18n("Play"), messages, SLOT(slotPlay()) );
  voice->insertItem( i18n("Stop"), messages, SLOT(slotStop()) );
  menubar->insertItem(i18n("Voice"), voice);
  
  // menu: fax
  QPopupMenu *fax = new QPopupMenu();
  fax->insertItem( i18n("New Fax"), this, SLOT(slotNewFax()) );
  fax->insertItem( i18n("Show queue"), this, SLOT(slotShowQueue()) );
  fax->insertItem( i18n("Send spooled..."), this, SLOT(slotSendSpooled()) );
  fax->insertItem( i18n("Show Fax"), messages, SLOT(slotShow()) );
  menubar->insertItem(i18n("Fax")    , fax);
  
  // menu: connect
  QPopupMenu *mconnect = new QPopupMenu();
  bool test =false;
  if(QFile::exists(kapp->kde_bindir() + "/kppp"))
    {
      mconnect->insertItem( i18n("&ppp connection"), this, SLOT(kpppSlot()) );
      test = true;
    }
  if(QFile::exists(kapp->kde_bindir() + "/kom"))
    {
      mconnect->insertItem( i18n("&modem terminal"), this, SLOT(komSlot()) );
      test = true;
    }
  if(test) 
    menubar->insertItem(i18n("&Connect"), mconnect);

  // menu: options
  QPopupMenu *options = new QPopupMenu();
  options->insertItem( i18n("&setup"), this, SLOT(setupSlot()) );
  menubar->insertItem(i18n("&Options"), options);

  menubar->insertSeparator(-1);

  // menu: help
  QPopupMenu *help = kapp->getHelpMenu(TRUE, aboutText);
  menubar->insertItem(i18n("&Help") , help);

  setMenu(menubar);
  menubar->show();
}

void MainWidget::setupToolBar()
{
  KIconLoader* loader = kapp->getIconLoader();

  toolBar = new KToolBar(this);
  toolBar->insertButton(loader->loadIcon("kvplay.xpm"), BUTTON_PLAY,
			SIGNAL(clicked()), messages,
			SLOT(slotPlay()), TRUE,
			i18n("Play message") );
  setPlay(FALSE);

  toolBar->insertButton(loader->loadIcon("kvstop.xpm"), BUTTON_STOP,
			SIGNAL(clicked()), messages,
			SLOT(slotStop()), TRUE,
			i18n("Stop playing") );
  setStop(FALSE);

  toolBar->insertButton(loader->loadIcon("kvcomment.xpm"), BUTTON_COMMENT,
			SIGNAL(clicked()), messages,
			SLOT(slotComment()), TRUE,
			i18n("Add a comment") );
  setComment(FALSE);

  toolBar->insertSeparator();

  toolBar->insertButton(loader->loadIcon("openfax.xpm"), BUTTON_SHOW,
			SIGNAL(clicked()), messages,
			SLOT(slotShow()), TRUE,
			i18n("Show Fax") );
  setShow(FALSE);

  toolBar->insertSeparator();

  toolBar->insertButton(loader->loadIcon("fileprint.xpm"), BUTTON_PRINT,
			SIGNAL(clicked()), messages,
			SLOT(slotPrint()), TRUE,
			i18n("Print Fax"));
  setPrint(FALSE);

  toolBar->insertButton(loader->loadIcon("filedel2.xpm"), BUTTON_DELETE,
			SIGNAL(clicked()), messages,
			SLOT(slotDelete()), TRUE,
			i18n("Delete Message or Fax"));
  setDelete(FALSE);

  toolBar->insertButton(loader->loadIcon("filenew.xpm"), BUTTON_NEW,
			SIGNAL(clicked()), this,
			SLOT(slotNewFax()), TRUE,
			i18n("Compose New Fax"));

  toolBar->insertSeparator();
  
  toolBar->insertButton(loader->loadIcon("modem_on.xpm"), BUTTON_GETTY,
			SIGNAL(toggled(bool)), this,
			SLOT(slotToggleGetty(bool)), TRUE,
			i18n("Turn answering machine on/off"));
  toolBar->setToggle(BUTTON_GETTY, true);
  if(!isSuid) toolBar->setItemEnabled(BUTTON_GETTY,false); 
  
  QFile file("/etc/nologin." + GETTYHANDLER->getDevice(true));
  if(!file.exists())
      toolBar->toggleButton(BUTTON_GETTY);

  toolBar->insertButton(loader->loadIcon("take_call.xpm"), BUTTON_TAKE,
			SIGNAL(clicked()), GETTYHANDLER,
			SLOT(slotTakeCall()), TRUE,
			i18n("Take over call"));

  addToolBar(toolBar);
}

void MainWidget::setupStatusBar()
{
  STATUSBAR = new KStatusBar( this );
  STATUSBAR->setBorderWidth(1);

  STATUSBAR->setInsertOrder( KStatusBar::RightToLeft );
  STATUSBAR->insertItem("                      ",ID_PLAY);
  STATUSBAR->changeItem(i18n("Idle"),ID_PLAY);
  
  STATUSBAR->insertItem("Modem:           ", ID_MODEM);
#ifdef BROKEN
  Modem *modem = new Modem(this);
  STATUSBAR->changeItem(modem->status(), ID_MODEM);
  delete modem;
#else
  STATUSBAR->changeItem("", ID_MODEM);
#endif

  setStatusBar( STATUSBAR );
  // show statusBar
  enableStatusBar( KStatusBar::Show );
}

void MainWidget::setupMessages()
{

  messages = new Messages(this, "messages");
  setView(messages,TRUE);
  messages->setFocus ();
  messages->show();
}


// SLOTS:

void MainWidget::slotGreeting()
{
  if(greetings != 0) return;
  greetings = new Greetings(i18n("Greeting Messages"), messages);
  connect(greetings,SIGNAL(finished()),this,SLOT(slotGreetingExited()) );
  greetings->show();
}

void MainWidget::slotGreetingExited()
{
  delete greetings;
  greetings = 0;
}

// Answering machine

void MainWidget::setPlay(bool stat) { 
  toolBar->setItemEnabled(BUTTON_PLAY,stat); 
}

void MainWidget::setStop(bool stat) { 
  toolBar->setItemEnabled(BUTTON_STOP,stat); 
}

void MainWidget::setComment(bool stat) {
  toolBar->setItemEnabled(BUTTON_COMMENT,stat); 
}

void MainWidget::setDelete(bool stat) { 
  toolBar->setItemEnabled(BUTTON_DELETE,stat); 
}

void MainWidget::setPrint(bool stat) { 
  toolBar->setItemEnabled(BUTTON_PRINT,stat); 
}

void MainWidget::setShow(bool stat) { 
  toolBar->setItemEnabled(BUTTON_SHOW,stat); 
}

// Fax

void MainWidget::slotNewFax()
{
  if(sendfax != 0) return;
  sendfax = new SendFax(i18n("Fax Composer"));
  connect(sendfax,SIGNAL(closed()), 
	  this, SLOT(slotSendExited()) );
}

void
MainWidget::slotShowQueue()
{
  new ShowQueue();
}

void 
MainWidget::slotSendSpooled()
{
  QString aStr;

  gcfg->setGroup("executables");
  aStr = gcfg->readEntry("faxrunq");
  if(aStr.isEmpty())
  {
    KMsgBox::message(0,i18n("KVoice warning"),
		     i18n("Could not find the faxrunq executable.\n"
			  "Cannot send the fax."),
		     KMsgBox::EXCLAMATION);
    return;
  }


  KProcess *proc = new KProcess();
  *proc << aStr;
  proc->start();
  connect(proc, SIGNAL(processExited(KProcess *)), 
	  this, SLOT(slotFaxIsSend(KProcess *)) );
}

void 
MainWidget::slotFaxIsSend(KProcess *proc)
{
  delete proc;

  // FIXME: should test if sending was sucessful
  
  KMsgBox::message(0,i18n("KVoice report"),
		   i18n("Finished sending the faxes"),
		   KMsgBox::EXCLAMATION);
}

void MainWidget::slotSendExited()
{
  delete sendfax;
  sendfax = 0;
}

// mconnect (terminal or ppp connection)
void MainWidget::kpppSlot()
{
  KProcess proc;
  proc.setExecutable("kppp");
  proc.start(KProcess::DontCare);
}

void MainWidget::komSlot()
{
  KProcess proc;
  proc.setExecutable("kom");
  proc.start(KProcess::DontCare);
}

// options

void MainWidget::setupSlot()
     // creates the setup dialog
{
  setupWindow = new Setup( 0, i18n("kvoice configuration") );
  connect(setupWindow,SIGNAL(applyButtonPressed()),
	  this,SLOT(setupFinishedSlot()) );

  setupWindow->show();
}

void MainWidget::setupFinishedSlot()
// deletes the setup dialog, after pressing the ok button.
{
  setupWindow->applySettings();
  delete setupWindow;
  messages->slotAdjustChanges();
}

void MainWidget::slotQuit()
{
  writeConfig();

  // killing of m/vgetty does not work otherwise
  GETTYHANDLER->quit();
  messages->quit();

  // remove tmp files
  gcfg->setGroup("location");
  QString aStr = gcfg->readEntry("tmpDir");
  aStr = "rm -f " + aStr;
  aStr += "/__kvoice*";
  system(aStr);
  
  dropPrivileges();
  cfg->sync();
  DATA->sync();
  kapp->quit();
}

void MainWidget::slotUnimplemented()
{
  KMsgBox::message(0,0,"Sorry, but this feature is\n"
		   "still unimplemented.",KMsgBox::EXCLAMATION);
}

void MainWidget::slotToggleGetty(bool on)
{
  if(on)
    GETTYHANDLER->on();
  else
    GETTYHANDLER->off();
}





