// $Id: setup.cpp,v 1.14 1998/09/06 20:41:27 lars Exp $
#include "setup.moc"

#include <unistd.h>
#include <assert.h>
#include <sys/types.h>
#include <signal.h>

 
#include <qfileinf.h>
#include <qmsgbox.h>
#include <qtstream.h>
#include <qlayout.h>
#include <qbttngrp.h>
#include <qradiobt.h>
#include <qchkbox.h>
#include <qlined.h>

#include <kintegerline.h>
#include <ksimpleconfig.h>
#include <kconfig.h>
#include <klocale.h>
#include <kapp.h>

#include "global.h"
#include "data.h"
#include "gettyhandler.h"

#include "defaults.h"

Setup *
Setup::setupobject = 0;

Setup::Setup(QWidget* parent, const char* name)
   : QTabDialog(parent, name)
{
  if(setupobject != 0) delete setupobject;
  setupobject = this;

  setCaption("setup");
  readConfig();
  setCancelButton();
  createDialog();

  connect(this, SIGNAL(applyButtonPressed()), SLOT(applySettings()) );

  // FIXME
  dirty = true;
}

Setup::~Setup()
{
  setupobject = 0;
}

Setup *
Setup::getSetup()
{
  if(setupobject == 0)
    setupobject = new Setup();
  return setupobject;
}

void
Setup::readConfig()
{
  QString str;

  gcfg->setGroup("location");
  spoolDir = gcfg->readEntry("spoolDir");
  voiceDir = gcfg->readEntry("voiceDir");
  voiceDir = voiceDir.right(voiceDir.size() - voiceDir.findRev("/") - 2);
  faxDir = gcfg->readEntry("faxDir");
  faxDir = faxDir.right(faxDir.size() - faxDir.findRev("/") - 2);
  mgettyDir = gcfg->readEntry("mgettyDir");
  tmpDir = gcfg->readEntry("tmpDir");

}

void
Setup::writeConfig(bool sync)
{
  QString aStr,tmpStr;

  cfg->setGroup("voice");
  if(buttonSoundCard->isChecked())
    cfg->writeEntry("device","SoundCard");
  else if(buttonHeadSet->isChecked())
    cfg->writeEntry("device","Headset");
  else
    cfg->writeEntry("device","Internal");
  if(sync)
    cfg->sync();

  cfg->setGroup("geometry");
  aStr.sprintf("%d %d", width(), height());
  cfg->writeEntry("setup", aStr);

  // privileged part
  if( isSuid && getuid() != 0 )
    return;
  gcfg->setGroup("location");
  gcfg->writeEntry("spoolDir", cSpoolDir->currentText() );
  aStr = cSpoolDir->currentText();
  aStr += "/"; 
  aStr += cVoiceDir->currentText();
  gcfg->writeEntry("voiceDir", aStr);

  //FIXME
  tmpStr = aStr + "/incoming";
  gcfg->writeEntry("voiceIncoming",tmpStr);
  tmpStr = aStr + "/messages";
  gcfg->writeEntry("voiceMessages",tmpStr);
  
  aStr = cSpoolDir->currentText();
  aStr +=  "/";
  aStr += cFaxDir->currentText();
  gcfg->writeEntry("faxDir", aStr );
  // FIXME
  tmpStr = aStr + "/incoming";
  gcfg->writeEntry("faxIncoming",tmpStr);
  tmpStr = aStr + "/outgoing";
  gcfg->writeEntry("faxOutgoing",tmpStr);

  gcfg->writeEntry("mgettyDir", cMgettyDir->currentText() );
  gcfg->writeEntry("tmpDir", cTmpDir->currentText() );

  gcfg->setGroup("modem");
  gcfg->writeEntry("type", cType->currentText() );
  gcfg->writeEntry("device", cPort->currentText() );

  cfg->setGroup("messages");
  cfg->writeEntry("checkInterval",checkInterval->value());

  gcfg->setGroup("kvoice");
  gcfg->writeEntry("configured","yes");

  if (sync)
    gcfg->sync();

}

void 
Setup::applySettings()
{
  QString aStr, oldAnswerMode;

  if(!dirty) return;

  writeConfig(true);

  // don't write back if the user doesn't have the permissions
  if(!DATA->readonly())
  {
    //use the changed configuration
    DATA->writeMgettyEntry( "speed", cSpeed->currentText());
    DATA->writeVoiceEntry( "port_speed", cSpeed->currentText());
    aStr.setNum(rings->value());
    DATA->writeMgettyEntry( "rings", aStr);
    DATA->writeVgettyEntry( "rings", aStr);
    aStr.setNum(beep_freq->value());
    DATA->writeVoiceEntry("beep_frequency", aStr);
    aStr.setNum(beep_len->value()/10);
    DATA->writeVoiceEntry("beep_length", aStr);
    DATA->writeMgettyEntry("fax-id", cFaxid->text());
    DATA->writeSendfaxEntry("fax-id", cFaxid->text());
    DATA->writeSendfaxEntry("fax-devices", cPort->currentText());

    // FIXME: we should switch to mgetty if voice is not desired
    aStr = "";
    oldAnswerMode = DATA->readVgettyEntry("answer_mode");
    if(buttonVoice->isChecked()) 
      aStr += "voice:";
    else
    {
      // it only makes sense to write these to mgetty.config,
      // if no voice operation is desired
      if(buttonFax->isChecked() && buttonData->isChecked())
      {
	DATA->writeMgettyEntry("data-only","no");
	DATA->writeMgettyEntry("fax-only","no");
      } 
      else if (buttonFax->isChecked())
      {
	DATA->writeMgettyEntry("data-only","no");
	DATA->writeMgettyEntry("fax-only","yes");
      }
      else if (buttonVoice->isChecked())
      {
	DATA->writeMgettyEntry("data-only","yes");
	DATA->writeMgettyEntry("fax-only","no");
      }
    }
    if(buttonFax->isChecked()) aStr += "fax:";
    if(buttonData->isChecked()) aStr += "data:";
    aStr.truncate(aStr.length()-1);
    DATA->writeVgettyEntry("answer_mode",aStr);
    
    aStr = cSpoolDir->currentText();
    aStr += "/"; 
    aStr += cVoiceDir->currentText();
    DATA->writeVoiceEntry("voice_dir",aStr);
    //FIXME
    DATA->writeVoiceEntry("receive_dir","incoming");
    DATA->writeVoiceEntry("message_dir","messages");

    DATA->writeBack();
  }

  // write new inittab if modem device changed
  if(GETTYHANDLER->getDevice() != cPort->currentText() 
     || !GETTYHANDLER->inittabEntryExists()
     || oldAnswerMode != DATA->readVgettyEntry("answer_mode"))
      GETTYHANDLER->writeInittab( cPort->currentText() );

  dirty = false;
}

void Setup::createDialog()
{
  int w,h;
  QString str;

  setMinimumSize(200,240);
  resize(200,240);

  setupGeneral();
  setupModem();
  setupVoice();
  setupFax();
  setupLocations();

//  adjustSize();
//  setMinimumSize(size());

  cfg->setGroup("geometry");
  str = cfg->readEntry("setup", "250 300");
  sscanf(str.data(), "%d %d", &w, &h);
  resize(w, h);

  show();
}

void Setup::setupModem()
{
  QString aStr;

  wProperties = new QWidget(this, "properties");
  QGridLayout *layout = new QGridLayout(wProperties, 6, 2, 4, 4);
  layout->setColStretch(0,1);
  layout->setColStretch(1,100);
  layout->setRowStretch(5,100);

  aStr = GETTYHANDLER->getDevice();
  cPort = addComboRow(wProperties, layout, i18n("modem device"),
		      Defaults::modemdevice, 
		      aStr,
		      0 );

  aStr = DATA->readMgettyEntry("speed");
  if(aStr.isEmpty())
    aStr = DATA->readVoiceEntry("speed");
  cSpeed = addComboRow(wProperties, layout, i18n("Baud Rate"),
		       Defaults::speed,  
		       aStr,
		       1 );

  gcfg->setGroup("modem");
  aStr = gcfg->readEntry("type");
  cType = addComboRow(wProperties, layout, i18n("modem Type"),
		      Defaults::modemtype, aStr,
		      2 );

  bool ok;
  int tmpint = DATA->readVgettyEntry("rings").toInt(&ok);
  if(!ok)
    tmpint = DATA->readMgettyEntry("rings").toInt(&ok);
  if(!ok)
    tmpint = 5;
  rings = addIntegerLine(wProperties, layout, 
			 i18n("rings before answering"),
			 tmpint, 3);

  QButtonGroup *aButtonGrp = new QButtonGroup(
    i18n("Answer to the following incoming calls:"), 
    wProperties);

  QGridLayout *layout2 = new QGridLayout(aButtonGrp, 3, 1, 20, 4);
  buttonFax = new QCheckBox(aButtonGrp);
  buttonFax->setText(i18n("Fax"));
  buttonFax->adjustSize();
  buttonFax->setMinimumSize(buttonFax->size());
  layout2->addWidget(buttonFax, 0, 0);

  buttonVoice = new QCheckBox(aButtonGrp);
  buttonVoice->setText(i18n("Voice"));
  buttonVoice->adjustSize();
  buttonVoice->setMinimumSize(buttonVoice->size());
  layout2->addWidget(buttonVoice, 1, 0);

  buttonData = new QCheckBox(aButtonGrp);
  buttonData->setText(i18n("Data"));
  buttonData->adjustSize();
  buttonData->setMinimumSize(buttonData->size());
  layout2->addWidget(buttonData, 2, 0);
  layout2->activate();

  aStr = DATA->readVgettyEntry("answer_mode");
  if(aStr.contains("fax"))
    buttonFax->setChecked(true);
  if(aStr.contains("voice"))
    buttonVoice->setChecked(true);
  if(aStr.contains("data"))
    buttonData->setChecked(true);

  aButtonGrp->adjustSize();
  aButtonGrp->setMinimumSize(aButtonGrp->size());

  layout->addMultiCellWidget(aButtonGrp,4,4,0,1);
  layout->activate();

// FIXME
//  wProperties->adjustSize();
//  wProperties->setMinimumSize(size());

  addTab(wProperties, i18n("Modem"));  
  if(isSuid && getuid() != 0)
    setTabEnabled("properties",false);

}


void
Setup::setupLocations()
{
  wLocations = new QWidget(this,"locations");
  QGridLayout *layout = new QGridLayout(wLocations, 6, 2, 4, 4);
  layout->setColStretch(0,1);
  layout->setColStretch(1,100);
  layout->setRowStretch(5,100);

  cSpoolDir = addComboRow(wLocations, layout, i18n("Spool Directory"),
			  Defaults::spooldir, 
			  spoolDir,
			  0 );
  
  cVoiceDir = addComboRow(wLocations, layout, i18n("  Voice Directory"),
			  Defaults::voicespool,
			  voiceDir,
			  1 );

  cFaxDir = addComboRow(wLocations, layout, i18n("  Fax Directory"),
			  Defaults::faxspool,
			  faxDir,
			  2 );

  cMgettyDir = addComboRow(wLocations, layout, i18n("mgetty Directory"),
			  Defaults::mgettydir,
			  mgettyDir,
			  3 );

  cTmpDir = addComboRow(wLocations, layout, i18n("tmp Directory"),
			  Defaults::tmpdir,
			  tmpDir,
			  4 );

  layout->activate();
  wLocations->adjustSize();
  wLocations->setMinimumSize(wLocations->size());

  addTab(wLocations, i18n("Directories"));
  if(isSuid && getuid() != 0)
    setTabEnabled("locations",false);  
}

void 
Setup::setupGeneral()
{
  wGeneral = new QWidget(this);

  QGridLayout *layout = new QGridLayout(wGeneral,3,2,4,4);
  layout->setColStretch(0,1);
  layout->setColStretch(1,100);
  layout->setRowStretch(1,1);
  layout->setRowStretch(2,100);

  QWidget *check = new QWidget(wGeneral);
  QGridLayout *layout2 = new QGridLayout(check, 1, 3, 0, 0);
  layout2->setColStretch(0,1);
  layout2->setColStretch(1,10);
  layout2->setColStretch(2,1);

  QLabel *text1 = new QLabel(check);
  text1->setText(i18n("Check for new messages every"));
  text1->adjustSize();
  text1->setMinimumSize(text1->size());
  layout2->addWidget(text1,0,0);

  checkInterval = new KIntegerLine(check);
  cfg->setGroup("messages");
  checkInterval->setValue(cfg->readNumEntry("checkInterval"));
  checkInterval->adjustSize();
  checkInterval->setMinimumSize(0,checkInterval->height());
  layout2->addWidget(checkInterval,0,1);
  
  QLabel *text2 = new QLabel(check);
  text2->setText(i18n(" minutes"));
  text2->adjustSize();
  text2->setMinimumSize(text2->size());
  check->adjustSize();

  layout2->addWidget(text2,0,2);
  layout2->activate();
  layout->addMultiCellWidget(check, 0, 0, 0, 1);

  layout->activate(); //FIXME

  addTab(wGeneral, i18n("General"));
  wGeneral->adjustSize();
}

void
Setup::setupVoice()
{
  wVoice = new QWidget(this);

  QGridLayout *layout2 = new QGridLayout(wVoice,4,2,4,4);
  layout2->setColStretch(0,1);
  layout2->setColStretch(1,100);
  layout2->setRowStretch(3,100);

  soundButtonGrp = new QButtonGroup(i18n("Play voice files through:"), 
				    wVoice);
  layout2->addMultiCellWidget(soundButtonGrp,0,0,0,1);

  QGridLayout *layout = new QGridLayout(soundButtonGrp, 3, 1, 20, 4);
  buttonSoundCard = new QRadioButton(soundButtonGrp);
  buttonSoundCard->setText(i18n("Soundcard"));
  buttonSoundCard->adjustSize();
  buttonSoundCard->setMinimumSize(buttonSoundCard->size());
  layout->addWidget(buttonSoundCard, 0, 0);

  buttonHeadSet = new QRadioButton(soundButtonGrp);
  buttonHeadSet->setText(i18n("Telephone line"));
  buttonHeadSet->adjustSize();
  buttonHeadSet->setMinimumSize(buttonHeadSet->size());
  layout->addWidget(buttonHeadSet, 1, 0);

  buttonInternal = new QRadioButton(soundButtonGrp);
  buttonInternal->setText(i18n("Modems internal speaker"));
  buttonInternal->adjustSize();
  buttonInternal->setMinimumSize(buttonHeadSet->size());
  layout->addWidget(buttonInternal, 2, 0);
  layout->activate();
  soundButtonGrp->adjustSize();
  soundButtonGrp->setMinimumSize(soundButtonGrp->size());

  cfg->setGroup("voice");
  if(cfg->readEntry("device") == "Internal")
    buttonInternal->setChecked(true);
  else if(cfg->readEntry("device") == "Headset")
    buttonHeadSet->setChecked(true);
  else
    buttonSoundCard->setChecked(true);

  bool ok;
  int tmpint = DATA->readVoiceEntry("beep_frequency").toInt(&ok);
    if(!ok) tmpint = 933;
  beep_freq = addIntegerLine(wVoice, layout2, 
			     i18n("beep frequency (Hz)"),
			     tmpint, 1);

  tmpint = DATA->readVoiceEntry("beep_length").toInt(&ok)*10;
    if(!ok) tmpint = 1500;
  beep_len = addIntegerLine(wVoice, layout2, 
			    i18n("beep length (ms)"),
			    tmpint, 2);
  layout2->activate();

  addTab(wVoice, i18n("Voice"));
  wVoice->adjustSize();
  
}

void 
Setup::setupFax()
{
  QString aStr;

  wFax = new QWidget(this);

  QGridLayout *layout = new QGridLayout(wFax, 2, 2, 4, 4);
  layout->setColStretch(0,1);
  layout->setColStretch(1,100);
  layout->setRowStretch(1,100);

  aStr = DATA->readMgettyEntry("fax-id");
  if(aStr.isEmpty())
    aStr = DATA->readSendfaxEntry("fax-id");
  cFaxid = addLineEdit(wFax, layout, i18n("phone number"),
		      aStr, 0 );


  addTab(wFax, i18n("Fax"));
  wVoice->adjustSize();

}

QComboBox *
Setup::addComboRow(QWidget *_parent, QGridLayout *_layout,
		   QString _label, const char *_items[], 
		   QString _current,
		   int _row)
{
  int i=0;

  QLabel *label = new QLabel(_parent);
  label->setText(_label);
  label->adjustSize();
  label->setMinimumSize(label->size());
  _layout->addWidget(label, _row, 0);
  
  QComboBox *combo = new QComboBox(true, _parent);
  combo->insertStrList(_items);
  if(!_current.isEmpty() )
    while(1)
    {
      if(_items[i] == _current)
      {
	combo->setCurrentItem(i);
	break;
      }
      if(_items[i] == 0 )
      {
	combo->insertItem( _current , 0);
	break;
      }
      i++;
    }
  combo->adjustSize();
  combo->setMinimumSize(combo->size());
  _layout->addWidget(combo, _row, 1);
  _layout->setRowStretch(_row,1);

  return combo;
}

QLineEdit *
Setup::addLineEdit(QWidget *_parent, QGridLayout *_layout,
		   QString _label,  
		   QString _current,
		   int _row)
{
  QLabel *label = new QLabel(_parent);
  label->setText(_label);
  label->adjustSize();
  label->setMinimumSize(label->size());
  _layout->addWidget(label, _row, 0);
  
  QLineEdit *edit = new QLineEdit(_parent);
  edit->setText(_current);
  edit->adjustSize();
  edit->setMinimumSize(edit->size());

  _layout->addWidget(edit, _row, 1);
  _layout->setRowStretch(_row,1);

  return edit;
}


KIntegerLine *
Setup::addIntegerLine(QWidget *_parent, QGridLayout *_layout,
		      QString _label,  
		      int _current,
		      int _row)
{
  QLabel *label = new QLabel(_parent);
  label->setText(_label);
  label->adjustSize();
  label->setMinimumSize(label->size());
  _layout->addWidget(label, _row, 0);
  
  KIntegerLine *edit = new KIntegerLine(_parent);
  edit->setValue(_current);
  edit->adjustSize();
//  edit->setMinimumSize(edit->size());

  _layout->addWidget(edit, _row, 1);
  _layout->setRowStretch(_row,1);

  return edit;
}








