// $Id: soundio.cpp,v 1.3 1998/06/24 08:19:35 knoll Exp $
#include <soundio.moc>
#include <global.h>

#include <qstring.h>

#include <kstatusbar.h>
#include <kapp.h>
#include <kprocess.h>
#include <kmsgbox.h>
#include <ksimpleconfig.h>

extern "C" {
#include <mediatool.h>
}
#include "kaudio.h"

#include "data.h"

SoundIO *SoundIO::thisObject = 0;

SoundIO::SoundIO()
{
  if(KAUDIO->serverStatus() == 0)
    connect(KAUDIO, SIGNAL(playFinished()),
	    this, SLOT(slotFinishedPlaying()) );

  thisObject = this;
  isplaying = false;
}

SoundIO::~SoundIO()
{
  thisObject = 0;
}

SoundIO *
SoundIO::getSoundIO()
{
  if (thisObject == 0)
    thisObject = new SoundIO;
  return thisObject;
}

bool SoundIO::play(QString filename)
{
  bool ok;
  if(isplaying) return false;

  cfg->setGroup("voice");
  QString device = cfg->readEntry("device");
  debug("device =%s\n",(const char *)device);

  if( device == "SoundCard" || device.isEmpty() )
  { 
    gcfg->setGroup("location");
    QString tmpfile = gcfg->readEntry("tmpdir") + "/__kvoice__convert.wav";
    convertToWav(filename, tmpfile);

    STATUSBAR->changeItem(i18n("Playing"),ID_PLAY);
    isplaying = TRUE;
    KAUDIO->play(strdup(tmpfile));
  }
  else
  {
    ok = true;
    if( device == "Headset")
    {
      ok = KMsgBox::yesNo( 0, i18n("Hear Message"), 
			   i18n("Lift up your telphone\n" 
				"to hear the message."), 
			   0, i18n("OK"), i18n("Cancel") );
    }
    if ( ok ) {
      proc = new KProcess();
      proc->setExecutable("vm");
      *proc << "play";
      if( device == "Headset")
	*proc << "-H";
      else // "Internal"
	*proc << "-s";
      *proc << filename;
      connect(proc, SIGNAL(processExited(KProcess *)),
	      this, SLOT(slotFinishedPlaying(KProcess *)) );
      proc->start(KProcess::NotifyOnExit);
      STATUSBAR->changeItem(i18n("Playing"),ID_PLAY);
      isplaying = TRUE;
    }
  } 

  if(isplaying)
  {
    emit playing();
    return true;
  }

  return false;
}

bool 
SoundIO::convertToWav(QString rmdfile, QString soundfile)
{
  gcfg->setGroup("location");
  QString tmpfile = gcfg->readEntry("tmpdir") + "/__kvoice__convert.pvf";

  KProcess proc;
  STATUSBAR->changeItem(i18n("Converting"),ID_PLAY); 
  kapp->processEvents();
  proc.setExecutable("rmdtopvf");
  proc << (const char *)rmdfile << (const char *)tmpfile;
  proc.start(KProcess::Block);
  proc.clearArguments();
  
  STATUSBAR->changeItem(i18n("Converting..."),ID_PLAY);
  kapp->processEvents();
  proc.setExecutable("pvftowav");
  proc << (const char *) tmpfile << (const char *) soundfile;
  proc.start(KProcess::Block);
}

bool
SoundIO::convertToRmd(QString soundfile, QString rmdfile)
{
  gcfg->setGroup("modem");
  if(gcfg->readEntry("type") == "UMC")
  {
        KMsgBox::message(0,i18n("Sorry"),
		     i18n("vgetty does not support conversion\n"
		       "to the UMC voice format.") );
	return false;
  }

  //get extension and base
  int index  = soundfile.findRev('.');
  QString extension = soundfile.right(soundfile.size() - index - 1);

  gcfg->setGroup("location");
  QString tmpfile = gcfg->readEntry("tmpdir") + "/__kvoice__wavtopvf.pvf";

  if(extension == ".wav") {
    KProcess proc;
    proc.setExecutable("wavtopvf");
    proc << (const char *) soundfile << tmpfile;
    proc.start(KProcess::Block);
  } else {
    KMsgBox::message(0,i18n("Error"),
		     i18n("Unknown file format") );
    return false;
  }
  
  KProcess proc;
  proc.setExecutable("pvftormd");
  proc << DATA->getModemType() << DATA->getCompression();
  proc << tmpfile << rmdfile;
  proc.start(KProcess::Block);

  return true;
}



void SoundIO::slotFinishedPlaying(KProcess *)
{
  delete proc;
  slotFinishedPlaying();
}

void SoundIO::slotFinishedPlaying()
{
  STATUSBAR->changeItem(i18n("Idle"),ID_PLAY);
  isplaying = FALSE;

  emit finished();
}

void SoundIO::slotStop()
{
  if (!isplaying) return;
  
  cfg->setGroup("voice");
  QString device = cfg->readEntry("device");
  if( device == "Headset" || device == 0)
  {
    debug("killing vm...\n");
    proc->kill(SIGKILL);
  }
  if( device == "SoundCard")
  {
    KAUDIO->stop();
  }

  STATUSBAR->changeItem(i18n("Idle"),ID_PLAY);
  isplaying = FALSE;

  emit finished();
}
